@@ -15,7 +15,7 @@ expr_reference <- function(names, table = NULL) {
   if (is.character(table) && !identical(table, "")) {
     names <- c(table, names)
   }
-   rapi_expr_reference(names)
+   rethrow_rapi_expr_reference(names)
 }
 
 #' Create a constant expression
@@ -44,7 +44,7 @@ expr_comparison <- rapi_expr_comparison
 #' @examples
 #' call_expr <- expr_function("ABS", list(expr_constant(-42)))
 expr_function <- function(name, args, order_bys = list(), filter_bys = list()) {
-  rapi_expr_function(name, args, order_bys, filter_bys)
+  rethrow_rapi_expr_function(name, args, order_bys, filter_bys)
 }
 
 #' Convert an expression to a string for debugging purposes
@@ -81,12 +81,12 @@ print.duckdb_expr <- function(x, ...) {
 #' con <- DBI::dbConnect(duckdb())
 #' rel <- rel_from_df(con, mtcars)
 rel_from_df <- function(con, df, experimental=FALSE) {
-    rapi_rel_from_df(con@conn_ref, as.data.frame(df), experimental)
+    rethrow_rapi_rel_from_df(con@conn_ref, as.data.frame(df), experimental)
 }
 
 #' @export
 print.duckdb_relation <- function(x, ...) {
-  message("DuckDB Relation: \n", rapi_rel_tostring(x))
+  message("DuckDB Relation: \n", rethrow_rapi_rel_tostring(x))
 }
 
 #' @export
@@ -94,17 +94,17 @@ as.data.frame.duckdb_relation <- function(x, row.names = NULL, optional = NULL,
   if (!missing(row.names) || !missing(optional)) {
     stop("row.names and optional parameters not supported")
   }
-  rapi_rel_to_df(x)
+  rethrow_rapi_rel_to_df(x)
 }
 
 #' @export
 names.duckdb_relation <- function(x) {
-  rapi_rel_names(x)
+  rethrow_rapi_rel_names(x)
 }
 
 #' @export
 head.duckdb_relation <- function(x, n = 6L, ...) {
-  rapi_rel_limit(x, n)
+  rethrow_rapi_rel_limit(x, n)
 }
 
 #' Lazily retrieve the top-n rows of a DuckDB relation object
@@ -116,7 +116,9 @@ head.duckdb_relation <- function(x, n = 6L, ...) {
 #' con <- DBI::dbConnect(duckdb())
 #' rel <- rel_from_df(con, mtcars)
 #' rel2 <- rel_limit(rel, 10)
-rel_limit <- rapi_rel_limit
+rel_limit <- function(rel, n) {
+  rethrow_rapi_rel_limit(rel, n)
+}
 
 #' Lazily project a DuckDB relation object
 #' @param rel the DuckDB relation object
@@ -127,7 +129,9 @@ rel_limit <- rapi_rel_limit
 #' con <- DBI::dbConnect(duckdb())
 #' rel <- rel_from_df(con, mtcars)
 #' rel2 <- rel_project(rel, list(expr_reference("cyl"), expr_reference("disp")))
-rel_project <- rapi_rel_project
+rel_project <- function(rel, exprs) {
+  rethrow_rapi_rel_project(rel, exprs)
+}
 
 #' Lazily filter a DuckDB relation object
 #' @param rel the DuckDB relation object
@@ -139,7 +143,9 @@ rel_project <- rapi_rel_project
 #' DBI::dbExecute(con, "CREATE OR REPLACE MACRO gt(a, b) AS a > b")
 #' rel <- rel_from_df(con, mtcars)
 #' rel2 <- rel_filter(rel, list(expr_function("gt", list(expr_reference("cyl"), expr_constant("6")))))
-rel_filter <- rapi_rel_filter
+rel_filter <- function(rel, exprs) {
+  rethrow_rapi_rel_filter(rel, exprs)
+}
 
 #' Lazily aggregate a DuckDB relation object
 #' @param rel the DuckDB relation object
@@ -152,7 +158,9 @@ rel_filter <- rapi_rel_filter
 #' rel <- rel_from_df(con, mtcars)
 #' aggrs <- list(avg_hp = expr_function("avg", list(expr_reference("hp"))))
 #' rel2 <- rel_aggregate(rel, list(expr_reference("cyl")), aggrs)
-rel_aggregate <- rapi_rel_aggregate
+rel_aggregate <- function(rel, groups, aggregates) {
+  rethrow_rapi_rel_aggregate(rel, groups, aggregates)
+}
 
 #' Lazily reorder a DuckDB relation object
 #' @param rel the DuckDB relation object
@@ -173,7 +181,7 @@ rel_order <- function(rel, orders, ascending = NULL) {
     stop("length of ascending must equal length of orders")
   }
 
-  return(rapi_rel_order(rel, orders, ascending))
+  return(rethrow_rapi_rel_order(rel, orders, ascending))
 }
 
 #' Get an external pointer pointing to NULL
@@ -215,7 +223,7 @@ expr_window_ <- function (window_function, partitions=list(), order_bys=list(),
           window_boundary_end=window_boundaries, start_expr = list(), end_expr=list(), offset_expr=list(), default_expr=list()) {
     window_boundary_start <- match.arg(window_boundary_start)
     window_boundary_end <- match.arg(window_boundary_end)
-    rapi_expr_window(window_function, partitions, order_bys, window_boundary_start, window_boundary_end, start_expr, end_expr, offset_expr, default_expr)
+    rethrow_rapi_expr_window(window_function, partitions, order_bys, window_boundary_start, window_boundary_end, start_expr, end_expr, offset_expr, default_expr)
 }
 
 #' Lazily INNER join two DuckDB relation objects
@@ -250,7 +258,7 @@ rel_join <- function(left, right, conds,
   if (join == "cross" && join_ref_type == "regular") {
     join_ref_type <- "cross"
   }
-  rapi_rel_join(left, right, conds, join, join_ref_type)
+  rethrow_rapi_rel_join(left, right, conds, join, join_ref_type)
 }
 
 #' UNION ALL on two DuckDB relation objects
@@ -263,7 +271,9 @@ rel_join <- function(left, right, conds,
 #' rel_a <- rel_from_df(con, mtcars)
 #' rel_b <- rel_from_df(con, mtcars)
 #' rel_union_all(rel_a, rel_b)
-rel_union_all <- rapi_rel_union_all
+rel_union_all <- function(rel_a, rel_b) {
+  rethrow_rapi_rel_union_all(rel_a, rel_b)
+}
 
 #' Lazily compute a distinct result on a DuckDB relation object
 #' @param rel the input DuckDB relation object
@@ -273,7 +283,9 @@ rel_union_all <- rapi_rel_union_all
 #' con <- DBI::dbConnect(duckdb())
 #' rel <- rel_from_df(con, mtcars)
 #' rel2 <- rel_distinct(rel)
-rel_distinct <- rapi_rel_distinct
+rel_distinct <- function(rel) {
+  rethrow_rapi_rel_distinct(rel)
+}
 
 #' SET INTERSECT on two DuckDB relation objects
 #' @param rel_a a DuckDB relation object
@@ -284,7 +296,9 @@ rel_distinct <- rapi_rel_distinct
 #' rel_a <- rel_from_df(con, mtcars)
 #' rel_b <- rel_from_df(con, mtcars)
 #' rel_set_intersect_all(rel_a, rel_b)
-rel_set_intersect <- rapi_rel_set_intersect
+rel_set_intersect <- function(rel_a, rel_b) {
+  rethrow_rapi_rel_set_intersect(rel_a, rel_b)
+}
 
 #' SET DIFF on two DuckDB relation objects
 #' @param rel_a a DuckDB relation object
@@ -295,7 +309,9 @@ rel_set_intersect <- rapi_rel_set_intersect
 #' rel_a <- rel_from_df(con, mtcars)
 #' rel_b <- rel_from_df(con, mtcars)
 #' rel_set_diff(rel_a, rel_b)
-rel_set_diff <- rapi_rel_set_diff
+rel_set_diff <- function(rel_a, rel_b) {
+  rethrow_rapi_rel_set_diff(rel_a, rel_b)
+}
 
 #' SET SYMDIFF on two DuckDB relation objects
 #' @param rel_a a DuckDB relation object
@@ -306,7 +322,9 @@ rel_set_diff <- rapi_rel_set_diff
 #' rel_a <- rel_from_df(con, mtcars)
 #' rel_b <- rel_from_df(con, mtcars)
 #' rel_set_symdiff(rel_a, rel_b)
-rel_set_symdiff <- rapi_rel_set_symdiff
+rel_set_symdiff <- function(rel_a, rel_b) {
+  rethrow_rapi_rel_set_symdiff(rel_a, rel_b)
+}
 
 #' Run a SQL query on a DuckDB relation object
 #' @param rel the DuckDB relation object
@@ -317,7 +335,9 @@ rel_set_symdiff <- rapi_rel_set_symdiff
 #' con <- DBI::dbConnect(duckdb())
 #' rel <- rel_from_df(con, mtcars)
 #' rel2 <- rel_sql(rel, "SELECT hp, cyl FROM _ WHERE hp > 100")
-rel_sql <- rapi_rel_sql
+rel_sql <- function(rel, sql) {
+  rethrow_rapi_rel_sql(rel, sql)
+}
 
 #' Print the EXPLAIN output for a DuckDB relation object
 #' @param rel the DuckDB relation object
@@ -327,7 +347,7 @@ rel_sql <- rapi_rel_sql
 #' rel <- rel_from_df(con, mtcars)
 #' rel_explain(rel)
 rel_explain <- function(rel) {
-  cat(rapi_rel_explain(rel)[[2]][[1]])
+  cat(rethrow_rapi_rel_explain(rel)[[2]][[1]])
   invisible(NULL)
 }
 
@@ -338,7 +358,9 @@ rel_explain <- function(rel) {
 #' con <- DBI::dbConnect(duckdb())
 #' rel <- rel_from_df(con, mtcars)
 #' rel_alias(rel)
-rel_alias <- rapi_rel_alias
+rel_alias <- function(rel) {
+  rethrow_rapi_rel_alias(rel)
+}
 
 #' Set the internal alias for a DuckDB relation object
 #' @param rel the DuckDB relation object
@@ -348,7 +370,9 @@ rel_alias <- rapi_rel_alias
 #' con <- DBI::dbConnect(duckdb())
 #' rel <- rel_from_df(con, mtcars)
 #' rel_set_alias(rel, "my_new_alias")
-rel_set_alias <- rapi_rel_set_alias
+rel_set_alias <- function(rel, alias) {
+  rethrow_rapi_rel_set_alias(rel, alias)
+}
 
 #' Transforms a relation object to a lazy data frame using altrep
 #' @param rel the DuckDB relation object
@@ -358,11 +382,18 @@ rel_set_alias <- rapi_rel_set_alias
 #' con <- DBI::dbConnect(duckdb())
 #' rel <- rel_from_df(con, mtcars)
 #' print(rel_to_altrep(rel))
-rel_to_altrep <- rapi_rel_to_altrep
+rel_to_altrep <- function(rel, allow_materialization = TRUE) {
+  rethrow_rapi_rel_to_altrep(rel, allow_materialization)
+}
 
 
 #' Retrieves the data frame back from a altrep df
 #' @param df the data frame created by rel_to_altrep
+#' @param strict whether to throw an error if the data frame is not an altrep
+#'   or if other criteria are not met
+#' @param allow_materialized whether to succeed if the data frame is already materialized
+#' @param enable_materialization set to `TRUE` for side effect: allow materialization of this a data frame
+#'   if it was not allowed previously
 #' @return the relation object
 #' @noRd
 #' @examples
@@ -370,8 +401,8 @@ rel_to_altrep <- rapi_rel_to_altrep
 #' rel <- rel_from_df(con, mtcars)
 #' df = rel_to_altrep(rel)
 #' print(rel_from_altrep_df(df))
-rel_from_altrep_df <- function(df, strict = TRUE, allow_materialized = TRUE) {
-  rapi_rel_from_altrep_df(df, strict, allow_materialized)
+rel_from_altrep_df <- function(df, strict = TRUE, allow_materialized = TRUE, enable_materialization = FALSE) {
+  rethrow_rapi_rel_from_altrep_df(df, strict, allow_materialized, enable_materialization)
 }
 
 
@@ -390,7 +421,9 @@ df_is_materialized <- function(df) {
 #' con <- DBI::dbConnect(duckdb())
 #' rel <- rel_from_df(con, mtcars)
 #' print(rel_to_sql(rel))
-rel_to_sql <- rapi_rel_to_sql
+rel_to_sql <- function(rel) {
+  rethrow_rapi_rel_to_sql(rel)
+}
 
 
 
@@ -403,7 +436,7 @@ rel_to_sql <- rapi_rel_to_sql
 #' DBI::dbWriteTable(con, "mtcars", mtcars)
 #' rel <- rel_from_sql(con, "SELECT * FROM mtcars")
 rel_from_sql <- function(con, sql) {
-    rapi_rel_from_sql(con@conn_ref, sql)
+    rethrow_rapi_rel_from_sql(con@conn_ref, sql)
 }
 
 #' Create a duckdb table relation from a table name
@@ -415,7 +448,7 @@ rel_from_sql <- function(con, sql) {
 #' DBI::dbWriteTable(con, "mtcars", mtcars)
 #' rel <- rel_from_table(con, "mtcars")
 rel_from_table <- function(con, table_name, schema_name = "MAIN") {
-    rapi_rel_from_table(con@conn_ref, schema_name, table_name)
+  rethrow_rapi_rel_from_table(con@conn_ref, schema_name, table_name)
 }
 
 #' Convert a duckdb relation from a table-producing function
@@ -428,8 +461,21 @@ rel_from_table <- function(con, table_name, schema_name = "MAIN") {
 #' con <- DBI::dbConnect(duckdb())
 #' rel <- rel_from_table_function(con, 'generate_series', list(10L))
 rel_from_table_function <- function(con, function_name, positional_parameters = list(), named_parameters = list()) {
-    rapi_rel_from_table_function(con@conn_ref, function_name, positional_parameters, named_parameters)
+  rethrow_rapi_rel_from_table_function(con@conn_ref, function_name, positional_parameters, named_parameters)
 }
 
-rel_to_parquet <- rapi_rel_to_parquet
+rel_to_parquet <- function(rel, file_name) {
+  rethrow_rapi_rel_to_parquet(rel, file_name)
+}
 
+rel_names <- function(rel) {
+  rethrow_rapi_rel_names(rel)
+}
+
+load_rfuns <- function() {
+  rethrow_rapi_load_rfuns()
+}
+
+get_last_rel <- function() {
+  rethrow_rapi_get_last_rel()
+}
