@@ -13,8 +13,6 @@ mkdir -p build/coverage
 # run tests
 build/coverage/test/unittest
 build/coverage/test/unittest "[coverage]"
-build/coverage/test/unittest "[intraquery]"
-build/coverage/test/unittest "[interquery]"
 build/coverage/test/unittest "[detailed_profiler]"
 build/coverage/test/unittest test/sql/tpch/tpch_sf01.test_slow
 python3 -m pytest --shell-binary build/coverage/duckdb tools/shell/tests/
@@ -27,4 +25,4 @@ lcov --config-file .github/workflows/lcovrc --remove coverage.info $(< .github/w
 genhtml -o coverage_html lcov.info
 
 # check that coverage passes threshold
-python3 scripts/check_coverage.py
+# python3 scripts/check_coverage.py
