@@ -3,6 +3,7 @@
 #include "duckdb/common/fast_mem.hpp"
 #include "duckdb/common/printer.hpp"
 #include "duckdb/common/row_operations/row_operations.hpp"
+#include "duckdb/common/type_visitor.hpp"
 #include "duckdb/common/types/row/tuple_data_allocator.hpp"
 
 #include <algorithm>
@@ -163,7 +164,7 @@ void TupleDataCollection::InitializeChunkState(TupleDataChunkState &chunk_state,
 
 	for (auto &col : column_ids) {
 		auto &type = types[col];
-		if (type.Contains(LogicalTypeId::ARRAY)) {
+		if (TypeVisitor::Contains(type, LogicalTypeId::ARRAY)) {
 			auto cast_type = ArrayType::ConvertToList(type);
 			chunk_state.cached_cast_vector_cache.push_back(
 			    make_uniq<VectorCache>(Allocator::DefaultAllocator(), cast_type));
@@ -226,15 +227,13 @@ static inline void ToUnifiedFormatInternal(TupleDataVectorFormat &format, Vector
 		auto &entries = StructVector::GetEntries(vector);
 		D_ASSERT(format.children.size() == entries.size());
 		for (idx_t struct_col_idx = 0; struct_col_idx < entries.size(); struct_col_idx++) {
-			ToUnifiedFormatInternal(reinterpret_cast<TupleDataVectorFormat &>(format.children[struct_col_idx]),
-			                        *entries[struct_col_idx], count);
+			ToUnifiedFormatInternal(format.children[struct_col_idx], *entries[struct_col_idx], count);
 		}
 		break;
 	}
 	case PhysicalType::LIST:
 		D_ASSERT(format.children.size() == 1);
-		ToUnifiedFormatInternal(reinterpret_cast<TupleDataVectorFormat &>(format.children[0]),
-		                        ListVector::GetEntry(vector), ListVector::GetListSize(vector));
+		ToUnifiedFormatInternal(format.children[0], ListVector::GetEntry(vector), ListVector::GetListSize(vector));
 		break;
 	case PhysicalType::ARRAY: {
 		D_ASSERT(format.children.size() == 1);
@@ -246,19 +245,20 @@ static inline void ToUnifiedFormatInternal(TupleDataVectorFormat &format, Vector
 		// How many list_entry_t's do we need to cover the whole child array?
 		// Make sure we round up so its all covered
 		auto child_array_total_size = ArrayVector::GetTotalSize(vector);
-		auto list_entry_t_count = MaxValue((child_array_total_size + array_size) / array_size, count);
+		auto list_entry_t_count =
+		    MaxValue((child_array_total_size + array_size) / array_size, format.unified.validity.TargetCount());
 
 		// Create list entries!
-		format.array_list_entries = make_uniq_array<list_entry_t>(list_entry_t_count);
+		format.array_list_entries = make_unsafe_uniq_array<list_entry_t>(list_entry_t_count);
 		for (idx_t i = 0; i < list_entry_t_count; i++) {
 			format.array_list_entries[i].length = array_size;
 			format.array_list_entries[i].offset = i * array_size;
 		}
 		format.unified.data = reinterpret_cast<data_ptr_t>(format.array_list_entries.get());
 
-		ToUnifiedFormatInternal(reinterpret_cast<TupleDataVectorFormat &>(format.children[0]),
-		                        ArrayVector::GetEntry(vector), count * array_size);
-	} break;
+		ToUnifiedFormatInternal(format.children[0], ArrayVector::GetEntry(vector), child_array_total_size);
+		break;
+	}
 	default:
 		break;
 	}
@@ -384,6 +384,17 @@ void TupleDataCollection::InitializeChunk(DataChunk &chunk) const {
 	chunk.Initialize(allocator->GetAllocator(), layout.GetTypes());
 }
 
+void TupleDataCollection::InitializeChunk(DataChunk &chunk, const vector<column_t> &columns) const {
+	vector<LogicalType> chunk_types(columns.size());
+	// keep the order of the columns
+	for (idx_t i = 0; i < columns.size(); i++) {
+		auto column_idx = columns[i];
+		D_ASSERT(column_idx < layout.ColumnCount());
+		chunk_types[i] = layout.GetTypes()[column_idx];
+	}
+	chunk.Initialize(allocator->GetAllocator(), chunk_types);
+}
+
 void TupleDataCollection::InitializeScanChunk(TupleDataScanState &state, DataChunk &chunk) const {
 	auto &column_ids = state.chunk_state.column_ids;
 	D_ASSERT(!column_ids.empty());
@@ -419,7 +430,7 @@ void TupleDataCollection::InitializeScan(TupleDataScanState &state, vector<colum
 	for (auto &col : column_ids) {
 		auto &type = layout.GetTypes()[col];
 
-		if (type.Contains(LogicalTypeId::ARRAY)) {
+		if (TypeVisitor::Contains(type, LogicalTypeId::ARRAY)) {
 			auto cast_type = ArrayType::ConvertToList(type);
 			chunk_state.cached_cast_vector_cache.push_back(
 			    make_uniq<VectorCache>(Allocator::DefaultAllocator(), cast_type));
@@ -525,14 +536,18 @@ void TupleDataCollection::ScanAtIndex(TupleDataPinState &pin_state, TupleDataChu
 	segment.allocator->InitializeChunkState(segment, pin_state, chunk_state, chunk_index, false);
 	result.Reset();
 
+	ResetCachedCastVectors(chunk_state, column_ids);
+	Gather(chunk_state.row_locations, *FlatVector::IncrementalSelectionVector(), chunk.count, column_ids, result,
+	       *FlatVector::IncrementalSelectionVector(), chunk_state.cached_cast_vectors);
+	result.SetCardinality(chunk.count);
+}
+
+void TupleDataCollection::ResetCachedCastVectors(TupleDataChunkState &chunk_state, const vector<column_t> &column_ids) {
 	for (idx_t i = 0; i < column_ids.size(); i++) {
 		if (chunk_state.cached_cast_vectors[i]) {
 			chunk_state.cached_cast_vectors[i]->ResetFromCache(*chunk_state.cached_cast_vector_cache[i]);
 		}
 	}
-	Gather(chunk_state.row_locations, *FlatVector::IncrementalSelectionVector(), chunk.count, column_ids, result,
-	       *FlatVector::IncrementalSelectionVector(), chunk_state.cached_cast_vectors);
-	result.SetCardinality(chunk.count);
 }
 
 // LCOV_EXCL_START
