@@ -8,16 +8,20 @@
 #include "duckdb/parallel/thread_context.hpp"
 #include "duckdb/planner/expression/bound_reference_expression.hpp"
 #include "duckdb/storage/data_table.hpp"
+#include "duckdb/storage/table/delete_state.hpp"
+#include "duckdb/storage/table/update_state.hpp"
 
 namespace duckdb {
 
 PhysicalUpdate::PhysicalUpdate(vector<LogicalType> types, TableCatalogEntry &tableref, DataTable &table,
                                vector<PhysicalIndex> columns, vector<unique_ptr<Expression>> expressions,
-                               vector<unique_ptr<Expression>> bound_defaults, idx_t estimated_cardinality,
+                               vector<unique_ptr<Expression>> bound_defaults,
+                               vector<unique_ptr<BoundConstraint>> bound_constraints, idx_t estimated_cardinality,
                                bool return_chunk)
     : PhysicalOperator(PhysicalOperatorType::UPDATE, std::move(types), estimated_cardinality), tableref(tableref),
       table(table), columns(std::move(columns)), expressions(std::move(expressions)),
-      bound_defaults(std::move(bound_defaults)), return_chunk(return_chunk) {
+      bound_defaults(std::move(bound_defaults)), bound_constraints(std::move(bound_constraints)),
+      return_chunk(return_chunk) {
 }
 
 //===--------------------------------------------------------------------===//
@@ -38,8 +42,9 @@ public:
 class UpdateLocalState : public LocalSinkState {
 public:
 	UpdateLocalState(ClientContext &context, const vector<unique_ptr<Expression>> &expressions,
-	                 const vector<LogicalType> &table_types, const vector<unique_ptr<Expression>> &bound_defaults)
-	    : default_executor(context, bound_defaults) {
+	                 const vector<LogicalType> &table_types, const vector<unique_ptr<Expression>> &bound_defaults,
+	                 const vector<unique_ptr<BoundConstraint>> &bound_constraints)
+	    : default_executor(context, bound_defaults), bound_constraints(bound_constraints) {
 		// initialize the update chunk
 		auto &allocator = Allocator::Get(context);
 		vector<LogicalType> update_types;
@@ -55,6 +60,23 @@ public:
 	DataChunk update_chunk;
 	DataChunk mock_chunk;
 	ExpressionExecutor default_executor;
+	unique_ptr<TableDeleteState> delete_state;
+	unique_ptr<TableUpdateState> update_state;
+	const vector<unique_ptr<BoundConstraint>> &bound_constraints;
+
+	TableDeleteState &GetDeleteState(DataTable &table, TableCatalogEntry &tableref, ClientContext &context) {
+		if (!delete_state) {
+			delete_state = table.InitializeDelete(tableref, context, bound_constraints);
+		}
+		return *delete_state;
+	}
+
+	TableUpdateState &GetUpdateState(DataTable &table, TableCatalogEntry &tableref, ClientContext &context) {
+		if (!update_state) {
+			update_state = table.InitializeUpdate(tableref, context, bound_constraints);
+		}
+		return *update_state;
+	}
 };
 
 SinkResultType PhysicalUpdate::Sink(ExecutionContext &context, DataChunk &chunk, OperatorSinkInput &input) const {
@@ -106,13 +128,14 @@ SinkResultType PhysicalUpdate::Sink(ExecutionContext &context, DataChunk &chunk,
 			// we need to slice here
 			update_chunk.Slice(sel, update_count);
 		}
-		table.Delete(tableref, context.client, row_ids, update_chunk.size());
+		auto &delete_state = lstate.GetDeleteState(table, tableref, context.client);
+		table.Delete(delete_state, context.client, row_ids, update_chunk.size());
 		// for the append we need to arrange the columns in a specific manner (namely the "standard table order")
 		mock_chunk.SetCardinality(update_chunk);
 		for (idx_t i = 0; i < columns.size(); i++) {
 			mock_chunk.data[columns[i].index].Reference(update_chunk.data[i]);
 		}
-		table.LocalAppend(tableref, context.client, mock_chunk);
+		table.LocalAppend(tableref, context.client, mock_chunk, bound_constraints);
 	} else {
 		if (return_chunk) {
 			mock_chunk.SetCardinality(update_chunk);
@@ -120,7 +143,8 @@ SinkResultType PhysicalUpdate::Sink(ExecutionContext &context, DataChunk &chunk,
 				mock_chunk.data[columns[i].index].Reference(update_chunk.data[i]);
 			}
 		}
-		table.Update(tableref, context.client, row_ids, columns, update_chunk);
+		auto &update_state = lstate.GetUpdateState(table, tableref, context.client);
+		table.Update(update_state, context.client, row_ids, columns, update_chunk);
 	}
 
 	if (return_chunk) {
@@ -137,15 +161,14 @@ unique_ptr<GlobalSinkState> PhysicalUpdate::GetGlobalSinkState(ClientContext &co
 }
 
 unique_ptr<LocalSinkState> PhysicalUpdate::GetLocalSinkState(ExecutionContext &context) const {
-	return make_uniq<UpdateLocalState>(context.client, expressions, table.GetTypes(), bound_defaults);
+	return make_uniq<UpdateLocalState>(context.client, expressions, table.GetTypes(), bound_defaults,
+	                                   bound_constraints);
 }
 
 SinkCombineResultType PhysicalUpdate::Combine(ExecutionContext &context, OperatorSinkCombineInput &input) const {
-	auto &state = input.local_state.Cast<UpdateLocalState>();
 	auto &client_profiler = QueryProfiler::Get(context.client);
-	context.thread.profiler.Flush(*this, state.default_executor, "default_executor", 1);
+	context.thread.profiler.Flush(*this);
 	client_profiler.Flush(context.thread.profiler);
-
 	return SinkCombineResultType::FINISHED;
 }
 
@@ -175,7 +198,7 @@ SourceResultType PhysicalUpdate::GetData(ExecutionContext &context, DataChunk &c
 	auto &g = sink_state->Cast<UpdateGlobalState>();
 	if (!return_chunk) {
 		chunk.SetCardinality(1);
-		chunk.SetValue(0, 0, Value::BIGINT(g.updated_count));
+		chunk.SetValue(0, 0, Value::BIGINT(NumericCast<int64_t>(g.updated_count)));
 		return SourceResultType::FINISHED;
 	}
 
