@@ -12,10 +12,11 @@ struct SetSelectionVectorSelect {
 	                               ValidityMask &input_validity, Vector &selection_entry, idx_t child_idx,
 	                               idx_t &target_offset, idx_t selection_offset, idx_t input_offset,
 	                               idx_t target_length) {
-		idx_t sel_idx = selection_entry.GetValue(selection_offset + child_idx).GetValue<int64_t>() - 1;
-		if (sel_idx < target_length) {
-			selection_vector.set_index(target_offset, input_offset + sel_idx);
-			if (!input_validity.RowIsValid(input_offset + sel_idx)) {
+		auto sel_idx = selection_entry.GetValue(selection_offset + child_idx).GetValue<int64_t>() - 1;
+		if (sel_idx >= 0 && sel_idx < UnsafeNumericCast<int64_t>(target_length)) {
+			auto sel_idx_unsigned = UnsafeNumericCast<idx_t>(sel_idx);
+			selection_vector.set_index(target_offset, input_offset + sel_idx_unsigned);
+			if (!input_validity.RowIsValid(input_offset + sel_idx_unsigned)) {
 				validity_mask.SetInvalid(target_offset);
 			}
 		} else {
@@ -44,6 +45,12 @@ struct SetSelectionVectorWhere {
 		if (!input_validity.RowIsValid(input_offset + child_idx)) {
 			validity_mask.SetInvalid(target_offset);
 		}
+
+		if (child_idx >= target_length) {
+			selection_vector.set_index(target_offset, 0);
+			validity_mask.SetInvalid(target_offset);
+		}
+
 		target_offset++;
 	}
 
