@@ -17,7 +17,8 @@ enum class UndoFlags : uint32_t { // far too big but aligned (TM)
 	CATALOG_ENTRY = 1,
 	INSERT_TUPLE = 2,
 	DELETE_TUPLE = 3,
-	UPDATE_TUPLE = 4
+	UPDATE_TUPLE = 4,
+	SEQUENCE_VALUE = 5
 };
 
 } // namespace duckdb
