@@ -72,7 +72,7 @@ public:
 	}
 	explicit VectorBuffer(idx_t data_size) : buffer_type(VectorBufferType::STANDARD_BUFFER) {
 		if (data_size > 0) {
-			data = make_unsafe_uniq_array<data_t>(data_size);
+			data = make_unsafe_uniq_array_uninitialized<data_t>(data_size);
 		}
 	}
 	explicit VectorBuffer(unsafe_unique_array<data_t> data_p)
@@ -199,12 +199,16 @@ public:
 	VectorFSSTStringBuffer();
 
 public:
-	void AddDecoder(buffer_ptr<void> &duckdb_fsst_decoder_p) {
+	void AddDecoder(buffer_ptr<void> &duckdb_fsst_decoder_p, const idx_t string_block_limit) {
 		duckdb_fsst_decoder = duckdb_fsst_decoder_p;
+		decompress_buffer.resize(string_block_limit + 1);
 	}
 	void *GetDecoder() {
 		return duckdb_fsst_decoder.get();
 	}
+	vector<unsigned char> &GetDecompressBuffer() {
+		return decompress_buffer;
+	}
 	void SetCount(idx_t count) {
 		total_string_count = count;
 	}
@@ -215,6 +219,7 @@ public:
 private:
 	buffer_ptr<void> duckdb_fsst_decoder;
 	idx_t total_string_count = 0;
+	vector<unsigned char> decompress_buffer;
 };
 
 class VectorStructBuffer : public VectorBuffer {
