@@ -9,7 +9,7 @@
 #pragma once
 
 #include "duckdb/common/common.hpp"
-#include "duckdb/common/types/row_layout.hpp"
+#include "duckdb/common/types/row/tuple_data_layout.hpp"
 #include "duckdb/common/types/vector.hpp"
 #include "duckdb/execution/operator/aggregate/aggregate_object.hpp"
 
@@ -27,7 +27,7 @@ protected:
 	Allocator &allocator;
 	BufferManager &buffer_manager;
 	//! A helper for managing offsets into the data buffers
-	RowLayout layout;
+	TupleDataLayout layout;
 	//! The types of the payload columns stored in the hashtable
 	vector<LogicalType> payload_types;
 	//! Intermediate structures and data for aggregate filters
