@@ -44,8 +44,6 @@ class MetadataManager {
 public:
 	//! The amount of metadata blocks per storage block
 	static constexpr const idx_t METADATA_BLOCK_COUNT = 64;
-	//! The size of metadata blocks
-	static constexpr const idx_t METADATA_BLOCK_SIZE = AlignValueFloor(Storage::BLOCK_SIZE / METADATA_BLOCK_COUNT);
 
 public:
 	MetadataManager(BlockManager &block_manager, BufferManager &buffer_manager);
@@ -58,8 +56,8 @@ public:
 	MetadataPointer FromDiskPointer(MetaBlockPointer pointer);
 	MetadataPointer RegisterDiskPointer(MetaBlockPointer pointer);
 
-	static BlockPointer ToBlockPointer(MetaBlockPointer meta_pointer);
-	static MetaBlockPointer FromBlockPointer(BlockPointer block_pointer);
+	static BlockPointer ToBlockPointer(MetaBlockPointer meta_pointer, const idx_t metadata_block_size);
+	static MetaBlockPointer FromBlockPointer(BlockPointer block_pointer, const idx_t metadata_block_size);
 
 	//! Flush all blocks to disk
 	void Flush();
@@ -68,11 +66,14 @@ public:
 	void ClearModifiedBlocks(const vector<MetaBlockPointer> &pointers);
 
 	vector<MetadataBlockInfo> GetMetadataInfo() const;
+	vector<shared_ptr<BlockHandle>> GetBlocks() const;
 	idx_t BlockCount();
 
 	void Write(WriteStream &sink);
 	void Read(ReadStream &source);
 
+	idx_t GetMetadataBlockSize() const;
+
 protected:
 	BlockManager &block_manager;
 	BufferManager &buffer_manager;
@@ -81,6 +82,7 @@ protected:
 
 protected:
 	block_id_t AllocateNewBlock();
+	block_id_t PeekNextBlockId();
 	block_id_t GetNextBlockId();
 
 	void AddBlock(MetadataBlock new_block, bool if_exists = false);
@@ -88,8 +90,4 @@ protected:
 	void ConvertToTransient(MetadataBlock &block);
 };
 
-//! Detect mismatching constant values
-static_assert(MetadataManager::METADATA_BLOCK_SIZE * MetadataManager::METADATA_BLOCK_COUNT <= Storage::BLOCK_SIZE,
-              "metadata block count exceeds total block alloc size");
-
 } // namespace duckdb
