@@ -14,6 +14,7 @@
 #include "duckdb/storage/table_io_manager.hpp"
 #include "duckdb/storage/write_ahead_log.hpp"
 #include "duckdb/storage/database_size.hpp"
+#include "duckdb/common/enums/checkpoint_type.hpp"
 
 namespace duckdb {
 class BlockManager;
@@ -22,6 +23,7 @@ class CheckpointWriter;
 class DatabaseInstance;
 class TransactionManager;
 class TableCatalogEntry;
+struct PersistentCollectionData;
 
 class StorageCommitState {
 public:
@@ -30,8 +32,29 @@ public:
 	virtual ~StorageCommitState() {
 	}
 
+	//! Revert the commit
+	virtual void RevertCommit() = 0;
 	// Make the commit persistent
 	virtual void FlushCommit() = 0;
+
+	virtual void AddRowGroupData(DataTable &table, idx_t start_index, idx_t count,
+	                             unique_ptr<PersistentCollectionData> row_group_data) = 0;
+	virtual optional_ptr<PersistentCollectionData> GetRowGroupData(DataTable &table, idx_t start_index,
+	                                                               idx_t &count) = 0;
+	virtual bool HasRowGroupData() {
+		return false;
+	}
+};
+
+struct CheckpointOptions {
+	CheckpointOptions()
+	    : wal_action(CheckpointWALAction::DONT_DELETE_WAL), action(CheckpointAction::CHECKPOINT_IF_REQUIRED),
+	      type(CheckpointType::FULL_CHECKPOINT) {
+	}
+
+	CheckpointWALAction wal_action;
+	CheckpointAction action;
+	CheckpointType type;
 };
 
 //! StorageManager is responsible for managing the physical storage of the
@@ -45,35 +68,45 @@ public:
 	static StorageManager &Get(AttachedDatabase &db);
 	static StorageManager &Get(Catalog &catalog);
 
-	//! Initialize a database or load an existing database from the given path
-	void Initialize(optional_ptr<ClientContext> context);
+	//! Initialize a database or load an existing database from the database file path. The block_alloc_size is
+	//! either set, or invalid. If invalid, then DuckDB defaults to the default_block_alloc_size (DBConfig),
+	//! or the file's block allocation size, if it is an existing database.
+	void Initialize(const optional_idx block_alloc_size);
 
 	DatabaseInstance &GetDatabase();
 	AttachedDatabase &GetAttached() {
 		return db;
 	}
 
-	//! Get the WAL of the StorageManager, returns nullptr if in-memory
-	optional_ptr<WriteAheadLog> GetWriteAheadLog();
+	//! Gets the size of the WAL, or zero, if there is no WAL.
+	idx_t GetWALSize();
+	//! Gets the WAL of the StorageManager, or nullptr, if there is no WAL.
+	optional_ptr<WriteAheadLog> GetWAL();
+	//! Deletes the WAL file, and resets the unique pointer.
+	void ResetWAL();
 
 	//! Returns the database file path
-	string GetDBPath() {
+	string GetDBPath() const {
 		return path;
 	}
+	bool IsLoaded() const {
+		return load_complete;
+	}
 	//! The path to the WAL, derived from the database file path
 	string GetWALPath();
 	bool InMemory();
 
 	virtual bool AutomaticCheckpoint(idx_t estimated_wal_bytes) = 0;
-	virtual unique_ptr<StorageCommitState> GenStorageCommitState(Transaction &transaction, bool checkpoint) = 0;
+	virtual unique_ptr<StorageCommitState> GenStorageCommitState(WriteAheadLog &wal) = 0;
 	virtual bool IsCheckpointClean(MetaBlockPointer checkpoint_id) = 0;
-	virtual void CreateCheckpoint(bool delete_wal = false, bool force_checkpoint = false) = 0;
+	virtual void CreateCheckpoint(CheckpointOptions options = CheckpointOptions()) = 0;
 	virtual DatabaseSize GetDatabaseSize() = 0;
 	virtual vector<MetadataBlockInfo> GetMetadataInfo() = 0;
 	virtual shared_ptr<TableIOManager> GetTableIOManager(BoundCreateTableInfo *info) = 0;
+	virtual BlockManager &GetBlockManager() = 0;
 
 protected:
-	virtual void LoadDatabase(optional_ptr<ClientContext> context = nullptr) = 0;
+	virtual void LoadDatabase(const optional_idx block_alloc_size) = 0;
 
 protected:
 	//! The database this storage manager belongs to
@@ -104,6 +137,7 @@ public:
 //! Stores database in a single file.
 class SingleFileStorageManager : public StorageManager {
 public:
+	SingleFileStorageManager() = delete;
 	SingleFileStorageManager(AttachedDatabase &db, string path, bool read_only);
 
 	//! The BlockManager to read/store meta information and data in blocks
@@ -113,14 +147,15 @@ public:
 
 public:
 	bool AutomaticCheckpoint(idx_t estimated_wal_bytes) override;
-	unique_ptr<StorageCommitState> GenStorageCommitState(Transaction &transaction, bool checkpoint) override;
+	unique_ptr<StorageCommitState> GenStorageCommitState(WriteAheadLog &wal) override;
 	bool IsCheckpointClean(MetaBlockPointer checkpoint_id) override;
-	void CreateCheckpoint(bool delete_wal, bool force_checkpoint) override;
+	void CreateCheckpoint(CheckpointOptions options) override;
 	DatabaseSize GetDatabaseSize() override;
 	vector<MetadataBlockInfo> GetMetadataInfo() override;
 	shared_ptr<TableIOManager> GetTableIOManager(BoundCreateTableInfo *info) override;
+	BlockManager &GetBlockManager() override;
 
 protected:
-	void LoadDatabase(optional_ptr<ClientContext> context = nullptr) override;
+	void LoadDatabase(const optional_idx block_alloc_size) override;
 };
 } // namespace duckdb
