@@ -13,6 +13,12 @@ ViewRelation::ViewRelation(const shared_ptr<ClientContext> &context, string sche
 	context->TryBindRelation(*this, this->columns);
 }
 
+ViewRelation::ViewRelation(const shared_ptr<ClientContext> &context, unique_ptr<TableRef> ref, const string &view_name)
+    : Relation(context, RelationType::VIEW_RELATION), view_name(view_name), premade_tableref(std::move(ref)) {
+	context->TryBindRelation(*this, this->columns);
+	premade_tableref->alias = view_name;
+}
+
 unique_ptr<QueryNode> ViewRelation::GetQueryNode() {
 	auto result = make_uniq<SelectNode>();
 	result->select_list.push_back(make_uniq<StarExpression>());
@@ -21,6 +27,9 @@ unique_ptr<QueryNode> ViewRelation::GetQueryNode() {
 }
 
 unique_ptr<TableRef> ViewRelation::GetTableRef() {
+	if (premade_tableref) {
+		return premade_tableref->Copy();
+	}
 	auto table_ref = make_uniq<BaseTableRef>();
 	table_ref->schema_name = schema_name;
 	table_ref->table_name = view_name;
@@ -28,6 +37,7 @@ unique_ptr<TableRef> ViewRelation::GetTableRef() {
 }
 
 string ViewRelation::GetAlias() {
+	D_ASSERT(!view_name.empty());
 	return view_name;
 }
 
