@@ -1,4 +1,10 @@
 #include "duckdb/optimizer/filter_pullup.hpp"
+
+#include "duckdb/common/enum_util.hpp"
+#include "duckdb/planner/expression/bound_comparison_expression.hpp"
+#include "duckdb/planner/operator/logical_any_join.hpp"
+#include "duckdb/planner/operator/logical_comparison_join.hpp"
+#include "duckdb/planner/operator/logical_cross_product.hpp"
 #include "duckdb/planner/operator/logical_join.hpp"
 
 namespace duckdb {
@@ -38,7 +44,12 @@ unique_ptr<LogicalOperator> FilterPullup::PullupJoin(unique_ptr<LogicalOperator>
 
 	switch (join.join_type) {
 	case JoinType::INNER:
-		return PullupInnerJoin(std::move(op));
+		if (op->type == LogicalOperatorType::LOGICAL_ASOF_JOIN) {
+			//	We can only move filters through the left side of AsOf joins.
+			return PullupFromLeft(std::move(op));
+		} else {
+			return PullupInnerJoin(std::move(op));
+		}
 	case JoinType::LEFT:
 	case JoinType::ANTI:
 	case JoinType::SEMI: {
@@ -55,7 +66,48 @@ unique_ptr<LogicalOperator> FilterPullup::PullupInnerJoin(unique_ptr<LogicalOper
 	if (op->type == LogicalOperatorType::LOGICAL_DELIM_JOIN) {
 		return op;
 	}
-	return PullupBothSide(std::move(op));
+	D_ASSERT(op->type == LogicalOperatorType::LOGICAL_COMPARISON_JOIN ||
+	         op->type == LogicalOperatorType::LOGICAL_ANY_JOIN);
+
+	// Get the filters from both sides of the join
+	op = PullupBothSide(std::move(op));
+	vector<unique_ptr<Expression>> expressions;
+	if (op->type == LogicalOperatorType::LOGICAL_FILTER) {
+		expressions = std::move(op->expressions);
+		op = std::move(op->children[0]);
+	} else if (!can_pullup) {
+		return op; // No filters from below, and we can't pullup, stop.
+	}
+
+	// Also extract the filters of the joins
+	switch (op->type) {
+	case LogicalOperatorType::LOGICAL_COMPARISON_JOIN: {
+		auto &comp_join = op->Cast<LogicalComparisonJoin>();
+		for (auto &cond : comp_join.conditions) {
+			expressions.push_back(
+			    make_uniq<BoundComparisonExpression>(cond.comparison, std::move(cond.left), std::move(cond.right)));
+		}
+		break;
+	}
+	case LogicalOperatorType::LOGICAL_ANY_JOIN: {
+		auto &any_join = op->Cast<LogicalAnyJoin>();
+		expressions.push_back(std::move(any_join.condition));
+		break;
+	}
+	default:
+		throw NotImplementedException("PullupInnerJoin for LogicalOperatorType::%s", EnumUtil::ToString(op->type));
+	}
+
+	// Convert to cross product
+	op = make_uniq<LogicalCrossProduct>(std::move(op->children[0]), std::move(op->children[1]));
+	if (can_pullup) {
+		for (auto &expr : expressions) {
+			filters_expr_pullup.push_back(std::move(expr));
+		}
+	} else {
+		op = GeneratePullupFilter(std::move(op), expressions);
+	}
+	return op;
 }
 
 unique_ptr<LogicalOperator> FilterPullup::PullupCrossProduct(unique_ptr<LogicalOperator> op) {
