@@ -11,47 +11,47 @@ OnEntryNotFound Transformer::TransformOnEntryNotFound(bool missing_ok) {
 }
 
 unique_ptr<AlterStatement> Transformer::TransformAlter(duckdb_libpgquery::PGAlterTableStmt &stmt) {
-	D_ASSERT(stmt.relation);
 
+	D_ASSERT(stmt.relation);
 	if (stmt.cmds->length != 1) {
 		throw ParserException("Only one ALTER command per statement is supported");
 	}
 
 	auto result = make_uniq<AlterStatement>();
-	auto qname = TransformQualifiedName(*stmt.relation);
+	auto qualified_name = TransformQualifiedName(*stmt.relation);
 
-	// first we check the type of ALTER
+	// Check the ALTER type.
 	for (auto c = stmt.cmds->head; c != nullptr; c = c->next) {
-		auto command = reinterpret_cast<duckdb_libpgquery::PGAlterTableCmd *>(lfirst(c));
-		AlterEntryData data(qname.catalog, qname.schema, qname.name, TransformOnEntryNotFound(stmt.missing_ok));
-		// TODO: Include more options for command->subtype
+
+		auto command = PGPointerCast<duckdb_libpgquery::PGAlterTableCmd>(c->data.ptr_value);
+		AlterEntryData data(qualified_name.catalog, qualified_name.schema, qualified_name.name,
+		                    TransformOnEntryNotFound(stmt.missing_ok));
+
 		switch (command->subtype) {
 		case duckdb_libpgquery::PG_AT_AddColumn: {
-			auto cdef = PGPointerCast<duckdb_libpgquery::PGColumnDef>(command->def);
-
+			auto column_def = PGPointerCast<duckdb_libpgquery::PGColumnDef>(command->def);
 			if (stmt.relkind != duckdb_libpgquery::PG_OBJECT_TABLE) {
 				throw ParserException("Adding columns is only supported for tables");
 			}
-			if (cdef->category == duckdb_libpgquery::COL_GENERATED) {
+			if (column_def->category == duckdb_libpgquery::COL_GENERATED) {
 				throw ParserException("Adding generated columns after table creation is not supported yet");
 			}
-			auto centry = TransformColumnDefinition(*cdef);
 
-			if (cdef->constraints) {
-				for (auto constr = cdef->constraints->head; constr != nullptr; constr = constr->next) {
-					auto constraint = TransformConstraint(constr, centry, 0);
+			auto column_entry = TransformColumnDefinition(*column_def);
+			if (column_def->constraints) {
+				for (auto constr = column_def->constraints->head; constr != nullptr; constr = constr->next) {
+					auto constraint = TransformConstraint(*constr, column_entry, 0);
 					if (!constraint) {
 						continue;
 					}
 					throw ParserException("Adding columns with constraints not yet supported");
 				}
 			}
-			result->info = make_uniq<AddColumnInfo>(std::move(data), std::move(centry), command->missing_ok);
+			result->info = make_uniq<AddColumnInfo>(std::move(data), std::move(column_entry), command->missing_ok);
 			break;
 		}
 		case duckdb_libpgquery::PG_AT_DropColumn: {
-			bool cascade = command->behavior == duckdb_libpgquery::PG_DROP_CASCADE;
-
+			auto cascade = command->behavior == duckdb_libpgquery::PG_DROP_CASCADE;
 			if (stmt.relkind != duckdb_libpgquery::PG_OBJECT_TABLE) {
 				throw ParserException("Dropping columns is only supported for tables");
 			}
@@ -60,7 +60,6 @@ unique_ptr<AlterStatement> Transformer::TransformAlter(duckdb_libpgquery::PGAlte
 		}
 		case duckdb_libpgquery::PG_AT_ColumnDefault: {
 			auto expr = TransformExpression(command->def);
-
 			if (stmt.relkind != duckdb_libpgquery::PG_OBJECT_TABLE) {
 				throw ParserException("Alter column's default is only supported for tables");
 			}
@@ -68,21 +67,21 @@ unique_ptr<AlterStatement> Transformer::TransformAlter(duckdb_libpgquery::PGAlte
 			break;
 		}
 		case duckdb_libpgquery::PG_AT_AlterColumnType: {
-			auto cdef = PGPointerCast<duckdb_libpgquery::PGColumnDef>(command->def);
-			auto column_definition = TransformColumnDefinition(*cdef);
-			unique_ptr<ParsedExpression> expr;
+			auto column_def = PGPointerCast<duckdb_libpgquery::PGColumnDef>(command->def);
+			auto column_entry = TransformColumnDefinition(*column_def);
 
+			unique_ptr<ParsedExpression> expr;
 			if (stmt.relkind != duckdb_libpgquery::PG_OBJECT_TABLE) {
 				throw ParserException("Alter column's type is only supported for tables");
 			}
-			if (cdef->raw_default) {
-				expr = TransformExpression(cdef->raw_default);
+			if (column_def->raw_default) {
+				expr = TransformExpression(column_def->raw_default);
 			} else {
-				auto colref = make_uniq<ColumnRefExpression>(command->name);
-				expr = make_uniq<CastExpression>(column_definition.Type(), std::move(colref));
+				auto col_ref = make_uniq<ColumnRefExpression>(command->name);
+				expr = make_uniq<CastExpression>(column_entry.Type(), std::move(col_ref));
 			}
-			result->info = make_uniq<ChangeColumnTypeInfo>(std::move(data), command->name, column_definition.Type(),
-			                                               std::move(expr));
+			result->info =
+			    make_uniq<ChangeColumnTypeInfo>(std::move(data), command->name, column_entry.Type(), std::move(expr));
 			break;
 		}
 		case duckdb_libpgquery::PG_AT_SetNotNull: {
@@ -98,7 +97,6 @@ unique_ptr<AlterStatement> Transformer::TransformAlter(duckdb_libpgquery::PGAlte
 			throw NotImplementedException("No support for that ALTER TABLE option yet!");
 		}
 	}
-
 	return result;
 }
 
