@@ -49,7 +49,7 @@ unique_ptr<SQLStatement> Transformer::TransformPragma(duckdb_libpgquery::PGPragm
 		break;
 	case duckdb_libpgquery::PG_PRAGMA_TYPE_ASSIGNMENT:
 		if (info.parameters.size() != 1) {
-			throw InternalException("PRAGMA statement with assignment should contain exactly one parameter");
+			throw ParserException("PRAGMA statement with assignment should contain exactly one parameter");
 		}
 		if (!info.named_parameters.empty()) {
 			throw InternalException("PRAGMA statement with assignment cannot have named parameters");
