@@ -1,10 +1,11 @@
 #include "duckdb/storage/block.hpp"
+
 #include "duckdb/common/assert.hpp"
 
 namespace duckdb {
 
-Block::Block(Allocator &allocator, block_id_t id)
-    : FileBuffer(allocator, FileBufferType::BLOCK, Storage::BLOCK_SIZE), id(id) {
+Block::Block(Allocator &allocator, const block_id_t id, const idx_t block_size)
+    : FileBuffer(allocator, FileBufferType::BLOCK, block_size), id(id) {
 }
 
 Block::Block(Allocator &allocator, block_id_t id, uint32_t internal_size)
