@@ -1,9 +1,9 @@
 #include "duckdb/storage/string_uncompressed.hpp"
 
 #include "duckdb/common/pair.hpp"
-#include "duckdb/storage/checkpoint/write_overflow_strings_to_disk.hpp"
-#include "duckdb/common/serializer/serializer.hpp"
 #include "duckdb/common/serializer/deserializer.hpp"
+#include "duckdb/common/serializer/serializer.hpp"
+#include "duckdb/storage/checkpoint/write_overflow_strings_to_disk.hpp"
 #include "duckdb/storage/table/column_data.hpp"
 
 namespace duckdb {
@@ -22,7 +22,8 @@ UncompressedStringSegmentState::~UncompressedStringSegmentState() {
 // Analyze
 //===--------------------------------------------------------------------===//
 struct StringAnalyzeState : public AnalyzeState {
-	StringAnalyzeState() : count(0), total_string_size(0), overflow_strings(0) {
+	explicit StringAnalyzeState(const CompressionInfo &info)
+	    : AnalyzeState(info), count(0), total_string_size(0), overflow_strings(0) {
 	}
 
 	idx_t count;
@@ -31,7 +32,8 @@ struct StringAnalyzeState : public AnalyzeState {
 };
 
 unique_ptr<AnalyzeState> UncompressedStringStorage::StringInitAnalyze(ColumnData &col_data, PhysicalType type) {
-	return make_uniq<StringAnalyzeState>();
+	CompressionInfo info(col_data.GetBlockManager().GetBlockSize());
+	return make_uniq<StringAnalyzeState>(info);
 }
 
 bool UncompressedStringStorage::StringAnalyze(AnalyzeState &state_p, Vector &input, idx_t count) {
@@ -46,7 +48,7 @@ bool UncompressedStringStorage::StringAnalyze(AnalyzeState &state_p, Vector &inp
 		if (vdata.validity.RowIsValid(idx)) {
 			auto string_size = data[idx].GetSize();
 			state.total_string_size += string_size;
-			if (string_size >= StringUncompressed::STRING_BLOCK_LIMIT) {
+			if (string_size >= StringUncompressed::GetStringBlockLimit(state.info.GetBlockSize())) {
 				state.overflow_strings++;
 			}
 		}
@@ -62,6 +64,19 @@ idx_t UncompressedStringStorage::StringFinalAnalyze(AnalyzeState &state_p) {
 //===--------------------------------------------------------------------===//
 // Scan
 //===--------------------------------------------------------------------===//
+void UncompressedStringInitPrefetch(ColumnSegment &segment, PrefetchState &prefetch_state) {
+	prefetch_state.AddBlock(segment.block);
+	auto segment_state = segment.GetSegmentState();
+	if (segment_state) {
+		auto &state = segment_state->Cast<UncompressedStringSegmentState>();
+		auto &block_manager = segment.GetBlockManager();
+		for (auto &block_id : state.on_disk_blocks) {
+			auto block_handle = state.GetHandle(block_manager, block_id);
+			prefetch_state.AddBlock(block_handle);
+		}
+	}
+}
+
 unique_ptr<SegmentScanState> UncompressedStringStorage::StringInitScan(ColumnSegment &segment) {
 	auto result = make_uniq<StringScanState>();
 	auto &buffer_manager = BufferManager::GetBufferManager(segment.db);
@@ -87,7 +102,7 @@ void UncompressedStringStorage::StringScanPartial(ColumnSegment &segment, Column
 
 	for (idx_t i = 0; i < scan_count; i++) {
 		// std::abs used since offsets can be negative to indicate big strings
-		uint32_t string_length = std::abs(base_data[start + i]) - std::abs(previous_offset);
+		auto string_length = UnsafeNumericCast<uint32_t>(std::abs(base_data[start + i]) - std::abs(previous_offset));
 		result_data[result_offset + i] =
 		    FetchStringFromDict(segment, dict, result, baseptr, base_data[start + i], string_length);
 		previous_offset = base_data[start + i];
@@ -133,9 +148,9 @@ void UncompressedStringStorage::StringFetchRow(ColumnSegment &segment, ColumnFet
 	uint32_t string_length;
 	if ((idx_t)row_id == 0) {
 		// edge case where this is the first string in the dict
-		string_length = std::abs(dict_offset);
+		string_length = NumericCast<uint32_t>(std::abs(dict_offset));
 	} else {
-		string_length = std::abs(dict_offset) - std::abs(base_data[row_id - 1]);
+		string_length = NumericCast<uint32_t>(std::abs(dict_offset) - std::abs(base_data[row_id - 1]));
 	}
 	result_data[result_idx] = FetchStringFromDict(segment, dict, result, baseptr, dict_offset, string_length);
 }
@@ -146,11 +161,10 @@ void UncompressedStringStorage::StringFetchRow(ColumnSegment &segment, ColumnFet
 struct SerializedStringSegmentState : public ColumnSegmentState {
 	SerializedStringSegmentState() {
 	}
-	explicit SerializedStringSegmentState(vector<block_id_t> blocks_p) : blocks(std::move(blocks_p)) {
+	explicit SerializedStringSegmentState(vector<block_id_t> blocks_p) {
+		blocks = std::move(blocks_p);
 	}
 
-	vector<block_id_t> blocks;
-
 	void Serialize(Serializer &serializer) const override {
 		serializer.WriteProperty(1, "overflow_blocks", blocks);
 	}
@@ -175,7 +189,7 @@ UncompressedStringStorage::StringInitSegment(ColumnSegment &segment, block_id_t
 	return std::move(result);
 }
 
-idx_t UncompressedStringStorage::FinalizeAppend(ColumnSegment &segment, SegmentStatistics &stats) {
+idx_t UncompressedStringStorage::FinalizeAppend(ColumnSegment &segment, SegmentStatistics &) {
 	auto &buffer_manager = BufferManager::GetBufferManager(segment.db);
 	auto handle = buffer_manager.Pin(segment.block);
 	auto dict = GetDictionary(segment, handle);
@@ -183,10 +197,13 @@ idx_t UncompressedStringStorage::FinalizeAppend(ColumnSegment &segment, SegmentS
 	// compute the total size required to store this segment
 	auto offset_size = DICTIONARY_HEADER_SIZE + segment.count * sizeof(int32_t);
 	auto total_size = offset_size + dict.size;
-	if (total_size >= COMPACTION_FLUSH_LIMIT) {
+
+	CompressionInfo info(segment.GetBlockManager().GetBlockSize());
+	if (total_size >= info.GetCompactionFlushLimit()) {
 		// the block is full enough, don't bother moving around the dictionary
 		return segment.SegmentSize();
 	}
+
 	// the block has space left: figure out how much space we can save
 	auto move_amount = segment.SegmentSize() - total_size;
 	// move the dictionary so it lines up exactly with the offsets
@@ -240,7 +257,7 @@ CompressionFunction StringUncompressed::GetFunction(PhysicalType data_type) {
 	                           UncompressedStringStorage::StringInitAppend, UncompressedStringStorage::StringAppend,
 	                           UncompressedStringStorage::FinalizeAppend, nullptr,
 	                           UncompressedStringStorage::SerializeState, UncompressedStringStorage::DeserializeState,
-	                           UncompressedStringStorage::CleanupState);
+	                           UncompressedStringStorage::CleanupState, UncompressedStringInitPrefetch);
 }
 
 //===--------------------------------------------------------------------===//
@@ -293,12 +310,13 @@ void UncompressedStringStorage::WriteStringMemory(ColumnSegment &segment, string
 	if (!state.head || state.head->offset + total_length >= state.head->size) {
 		// string does not fit, allocate space for it
 		// create a new string block
-		idx_t alloc_size = MaxValue<idx_t>(total_length, Storage::BLOCK_SIZE);
+		auto alloc_size = MaxValue<idx_t>(total_length, segment.GetBlockManager().GetBlockSize());
 		auto new_block = make_uniq<StringBlock>();
 		new_block->offset = 0;
 		new_block->size = alloc_size;
 		// allocate an in-memory buffer for it
-		handle = buffer_manager.Allocate(MemoryTag::OVERFLOW_STRINGS, alloc_size, false, &block);
+		handle = buffer_manager.Allocate(MemoryTag::OVERFLOW_STRINGS, alloc_size, false);
+		block = handle.GetBlockHandle();
 		state.overflow_blocks.insert(make_pair(block->BlockId(), reference<StringBlock>(*new_block)));
 		new_block->block = std::move(block);
 		new_block->next = std::move(state.head);
@@ -321,12 +339,13 @@ void UncompressedStringStorage::WriteStringMemory(ColumnSegment &segment, string
 
 string_t UncompressedStringStorage::ReadOverflowString(ColumnSegment &segment, Vector &result, block_id_t block,
                                                        int32_t offset) {
-	D_ASSERT(block != INVALID_BLOCK);
-	D_ASSERT(offset < int32_t(Storage::BLOCK_SIZE));
-
 	auto &block_manager = segment.GetBlockManager();
 	auto &buffer_manager = block_manager.buffer_manager;
 	auto &state = segment.GetSegmentState()->Cast<UncompressedStringSegmentState>();
+
+	D_ASSERT(block != INVALID_BLOCK);
+	D_ASSERT(offset < NumericCast<int32_t>(block_manager.GetBlockSize()));
+
 	if (block < MAXIMUM_BLOCK) {
 		// read the overflow string from disk
 		// pin the initial handle and read the length
@@ -339,7 +358,7 @@ string_t UncompressedStringStorage::ReadOverflowString(ColumnSegment &segment, V
 		offset += sizeof(uint32_t);
 
 		// allocate a buffer to store the string
-		auto alloc_size = MaxValue<idx_t>(Storage::BLOCK_SIZE, length);
+		auto alloc_size = MaxValue<idx_t>(block_manager.GetBlockSize(), length);
 		// allocate a buffer to store the compressed string
 		// TODO: profile this to check if we need to reuse buffer
 		auto target_handle = buffer_manager.Allocate(MemoryTag::OVERFLOW_STRINGS, alloc_size);
@@ -347,10 +366,11 @@ string_t UncompressedStringStorage::ReadOverflowString(ColumnSegment &segment, V
 
 		// now append the string to the single buffer
 		while (remaining > 0) {
-			idx_t to_write = MinValue<idx_t>(remaining, Storage::BLOCK_SIZE - sizeof(block_id_t) - offset);
+			idx_t to_write = MinValue<idx_t>(remaining, block_manager.GetBlockSize() - sizeof(block_id_t) -
+			                                                UnsafeNumericCast<idx_t>(offset));
 			memcpy(target_ptr, handle.Ptr() + offset, to_write);
 			remaining -= to_write;
-			offset += to_write;
+			offset += UnsafeNumericCast<int32_t>(to_write);
 			target_ptr += to_write;
 			if (remaining > 0) {
 				// read the next block
@@ -364,16 +384,16 @@ string_t UncompressedStringStorage::ReadOverflowString(ColumnSegment &segment, V
 		auto final_buffer = target_handle.Ptr();
 		StringVector::AddHandle(result, std::move(target_handle));
 		return ReadString(final_buffer, 0, length);
-	} else {
-		// read the overflow string from memory
-		// first pin the handle, if it is not pinned yet
-		auto entry = state.overflow_blocks.find(block);
-		D_ASSERT(entry != state.overflow_blocks.end());
-		auto handle = buffer_manager.Pin(entry->second.get().block);
-		auto final_buffer = handle.Ptr();
-		StringVector::AddHandle(result, std::move(handle));
-		return ReadStringWithLength(final_buffer, offset);
 	}
+
+	// read the overflow string from memory
+	// first pin the handle, if it is not pinned yet
+	auto entry = state.overflow_blocks.find(block);
+	D_ASSERT(entry != state.overflow_blocks.end());
+	auto handle = buffer_manager.Pin(entry->second.get().block);
+	auto final_buffer = handle.Ptr();
+	StringVector::AddHandle(result, std::move(handle));
+	return ReadStringWithLength(final_buffer, offset);
 }
 
 string_t UncompressedStringStorage::ReadString(data_ptr_t target, int32_t offset, uint32_t string_length) {
@@ -401,44 +421,45 @@ void UncompressedStringStorage::ReadStringMarker(data_ptr_t target, block_id_t &
 	memcpy(&offset, target, sizeof(int32_t));
 }
 
-string_location_t UncompressedStringStorage::FetchStringLocation(StringDictionaryContainer dict, data_ptr_t baseptr,
-                                                                 int32_t dict_offset) {
-	D_ASSERT(dict_offset >= -1 * int32_t(Storage::BLOCK_SIZE) && dict_offset <= int32_t(Storage::BLOCK_SIZE));
-	if (dict_offset < 0) {
-		string_location_t result;
-		ReadStringMarker(baseptr + dict.end - idx_t(-1 * dict_offset), result.block_id, result.offset);
-		return result;
-	} else {
+string_location_t UncompressedStringStorage::FetchStringLocation(StringDictionaryContainer dict, data_ptr_t base_ptr,
+                                                                 int32_t dict_offset, const idx_t block_size) {
+	D_ASSERT(dict_offset + NumericCast<int32_t>(block_size) >= 0 && dict_offset <= NumericCast<int32_t>(block_size));
+	if (dict_offset >= 0) {
 		return string_location_t(INVALID_BLOCK, dict_offset);
 	}
+
+	string_location_t result;
+	ReadStringMarker(base_ptr + dict.end - NumericCast<idx_t>(-1 * dict_offset), result.block_id, result.offset);
+	return result;
 }
 
 string_t UncompressedStringStorage::FetchStringFromDict(ColumnSegment &segment, StringDictionaryContainer dict,
-                                                        Vector &result, data_ptr_t baseptr, int32_t dict_offset,
+                                                        Vector &result, data_ptr_t base_ptr, int32_t dict_offset,
                                                         uint32_t string_length) {
-	// fetch base data
-	D_ASSERT(dict_offset <= int32_t(Storage::BLOCK_SIZE));
-	string_location_t location = FetchStringLocation(dict, baseptr, dict_offset);
-	return FetchString(segment, dict, result, baseptr, location, string_length);
+	// Fetch the base data.
+	auto block_size = segment.GetBlockManager().GetBlockSize();
+	D_ASSERT(dict_offset <= NumericCast<int32_t>(block_size));
+	string_location_t location = FetchStringLocation(dict, base_ptr, dict_offset, block_size);
+	return FetchString(segment, dict, result, base_ptr, location, string_length);
 }
 
 string_t UncompressedStringStorage::FetchString(ColumnSegment &segment, StringDictionaryContainer dict, Vector &result,
-                                                data_ptr_t baseptr, string_location_t location,
+                                                data_ptr_t base_ptr, string_location_t location,
                                                 uint32_t string_length) {
 	if (location.block_id != INVALID_BLOCK) {
 		// big string marker: read from separate block
 		return ReadOverflowString(segment, result, location.block_id, location.offset);
-	} else {
-		if (location.offset == 0) {
-			return string_t(nullptr, 0);
-		}
-		// normal string: read string from this block
-		auto dict_end = baseptr + dict.end;
-		auto dict_pos = dict_end - location.offset;
-
-		auto str_ptr = char_ptr_cast(dict_pos);
-		return string_t(str_ptr, string_length);
 	}
+	if (location.offset == 0) {
+		return string_t(nullptr, 0);
+	}
+
+	// normal string: read string from this block
+	auto dict_end = base_ptr + dict.end;
+	auto dict_pos = dict_end - location.offset;
+
+	auto str_ptr = char_ptr_cast(dict_pos);
+	return string_t(str_ptr, string_length);
 }
 
 } // namespace duckdb
