@@ -1,24 +1,13 @@
 #include "duckdb/storage/statistics/numeric_stats.hpp"
-#include "duckdb/storage/statistics/base_statistics.hpp"
-#include "duckdb/common/types/vector.hpp"
-#include "duckdb/common/operator/comparison_operators.hpp"
 
-#include "duckdb/common/serializer/serializer.hpp"
+#include "duckdb/common/operator/comparison_operators.hpp"
 #include "duckdb/common/serializer/deserializer.hpp"
+#include "duckdb/common/serializer/serializer.hpp"
+#include "duckdb/common/types/vector.hpp"
+#include "duckdb/storage/statistics/base_statistics.hpp"
 
 namespace duckdb {
 
-template <>
-void NumericStats::Update<interval_t>(BaseStatistics &stats, interval_t new_value) {
-}
-
-template <>
-void NumericStats::Update<list_entry_t>(BaseStatistics &stats, list_entry_t new_value) {
-}
-
-//===--------------------------------------------------------------------===//
-// NumericStats
-//===--------------------------------------------------------------------===//
 BaseStatistics NumericStats::CreateUnknown(LogicalType type) {
 	BaseStatistics result(std::move(type));
 	result.InitializeUnknown();
@@ -98,6 +87,11 @@ hugeint_t GetNumericValueUnion::Operation(const NumericValueUnion &v) {
 	return v.value_.hugeint;
 }
 
+template <>
+uhugeint_t GetNumericValueUnion::Operation(const NumericValueUnion &v) {
+	return v.value_.uhugeint;
+}
+
 template <>
 uint8_t GetNumericValueUnion::Operation(const NumericValueUnion &v) {
 	return v.value_.utinyint;
@@ -248,6 +242,8 @@ FilterPropagateResult NumericStats::CheckZonemap(const BaseStatistics &stats, Ex
 		return CheckZonemapTemplated<uint64_t>(stats, comparison_type, constant);
 	case PhysicalType::INT128:
 		return CheckZonemapTemplated<hugeint_t>(stats, comparison_type, constant);
+	case PhysicalType::UINT128:
+		return CheckZonemapTemplated<uhugeint_t>(stats, comparison_type, constant);
 	case PhysicalType::FLOAT:
 		return CheckZonemapTemplated<float>(stats, comparison_type, constant);
 	case PhysicalType::DOUBLE:
@@ -301,6 +297,9 @@ void SetNumericValueInternal(const Value &input, const LogicalType &type, Numeri
 	case PhysicalType::INT128:
 		val.value_.hugeint = HugeIntValue::Get(input);
 		break;
+	case PhysicalType::UINT128:
+		val.value_.uhugeint = UhugeIntValue::Get(input);
+		break;
 	case PhysicalType::FLOAT:
 		val.value_.float_ = FloatValue::Get(input);
 		break;
@@ -344,6 +343,8 @@ Value NumericValueUnionToValueInternal(const LogicalType &type, const NumericVal
 		return Value::UBIGINT(val.value_.ubigint);
 	case PhysicalType::INT128:
 		return Value::HUGEINT(val.value_.hugeint);
+	case PhysicalType::UINT128:
+		return Value::UHUGEINT(val.value_.uhugeint);
 	case PhysicalType::FLOAT:
 		return Value::FLOAT(val.value_.float_);
 	case PhysicalType::DOUBLE:
@@ -442,6 +443,9 @@ static void SerializeNumericStatsValue(const LogicalType &type, NumericValueUnio
 	case PhysicalType::INT128:
 		serializer.WriteProperty(101, "value", val.value_.hugeint);
 		break;
+	case PhysicalType::UINT128:
+		serializer.WriteProperty(101, "value", val.value_.uhugeint);
+		break;
 	case PhysicalType::FLOAT:
 		serializer.WriteProperty(101, "value", val.value_.float_);
 		break;
@@ -492,6 +496,9 @@ static void DeserializeNumericStatsValue(const LogicalType &type, NumericValueUn
 	case PhysicalType::INT128:
 		result.value_.hugeint = deserializer.ReadProperty<hugeint_t>(101, "value");
 		break;
+	case PhysicalType::UINT128:
+		result.value_.uhugeint = deserializer.ReadProperty<uhugeint_t>(101, "value");
+		break;
 	case PhysicalType::FLOAT:
 		result.value_.float_ = deserializer.ReadProperty<float>(101, "value");
 		break;
@@ -587,6 +594,9 @@ void NumericStats::Verify(const BaseStatistics &stats, Vector &vector, const Sel
 	case PhysicalType::INT128:
 		TemplatedVerify<hugeint_t>(stats, vector, sel, count);
 		break;
+	case PhysicalType::UINT128:
+		TemplatedVerify<uhugeint_t>(stats, vector, sel, count);
+		break;
 	case PhysicalType::FLOAT:
 		TemplatedVerify<float>(stats, vector, sel, count);
 		break;
