@@ -32,6 +32,10 @@ static bool UseVersion(TransactionData transaction, transaction_t id) {
 	return TransactionVersionOperator::UseInsertedVersion(transaction.start_time, transaction.transaction_id, id);
 }
 
+bool ChunkInfo::Cleanup(transaction_t lowest_transaction, unique_ptr<ChunkInfo> &result) const {
+	return false;
+}
+
 void ChunkInfo::Write(WriteStream &writer) const {
 	writer.Write<ChunkInfoType>(type);
 }
@@ -95,6 +99,18 @@ idx_t ChunkConstantInfo::GetCommittedDeletedCount(idx_t max_count) {
 	return delete_id < TRANSACTION_ID_START ? max_count : 0;
 }
 
+bool ChunkConstantInfo::Cleanup(transaction_t lowest_transaction, unique_ptr<ChunkInfo> &result) const {
+	if (delete_id != NOT_DELETED_ID) {
+		// the chunk info is labeled as deleted - we need to keep it around
+		return false;
+	}
+	if (insert_id > lowest_transaction) {
+		// there are still transactions active that need this ChunkInfo
+		return false;
+	}
+	return true;
+}
+
 void ChunkConstantInfo::Write(WriteStream &writer) const {
 	D_ASSERT(HasDeletes());
 	ChunkInfo::Write(writer);
@@ -233,6 +249,28 @@ void ChunkVectorInfo::CommitAppend(transaction_t commit_id, idx_t start, idx_t e
 	}
 }
 
+bool ChunkVectorInfo::Cleanup(transaction_t lowest_transaction, unique_ptr<ChunkInfo> &result) const {
+	if (any_deleted) {
+		// if any rows are deleted we can't clean-up
+		return false;
+	}
+	// check if the insertion markers have to be used by all transactions going forward
+	if (!same_inserted_id) {
+		for (idx_t idx = 1; idx < STANDARD_VECTOR_SIZE; idx++) {
+			if (inserted[idx] > lowest_transaction) {
+				// transaction was inserted after the lowest transaction start
+				// we still need to use an older version - cannot compress
+				return false;
+			}
+		}
+	} else if (insert_id > lowest_transaction) {
+		// transaction was inserted after the lowest transaction start
+		// we still need to use an older version - cannot compress
+		return false;
+	}
+	return true;
+}
+
 bool ChunkVectorInfo::HasDeletes() const {
 	return any_deleted;
 }
