@@ -6,12 +6,16 @@
 
 #include "src/common/box_renderer.cpp"
 
+#include "src/common/cgroups.cpp"
+
 #include "src/common/compressed_file_system.cpp"
 
 #include "src/common/constants.cpp"
 
 #include "src/common/checksum.cpp"
 
+#include "src/common/encryption_state.cpp"
+
 #include "src/common/exception.cpp"
 
 #include "src/common/exception_format_value.cpp"
@@ -30,7 +34,7 @@
 
 #include "src/common/hive_partitioning.cpp"
 
-#include "src/common/http_state.cpp"
+#include "src/common/http_util.cpp"
 
 #include "src/common/pipe_file_system.cpp"
 
@@ -54,6 +58,8 @@
 
 #include "src/common/enum_util.cpp"
 
+#include "src/common/render_tree.cpp"
+
 #include "src/common/tree_renderer.cpp"
 
 #include "src/common/types.cpp"
