@@ -1,5 +1,9 @@
+#include "src/core_functions/aggregate/holistic/approx_top_k.cpp"
+
 #include "src/core_functions/aggregate/holistic/quantile.cpp"
 
+#include "src/core_functions/aggregate/holistic/mad.cpp"
+
 #include "src/core_functions/aggregate/holistic/mode.cpp"
 
 #include "src/core_functions/aggregate/holistic/approximate_quantile.cpp"
