@@ -218,3 +218,17 @@ cpp11::r_string rapi_ptr_to_str(SEXP extptr);
 void duckdb_r_transform(duckdb::Vector &src_vec, SEXP dest, duckdb::idx_t dest_offset, duckdb::idx_t n, bool integer64);
 SEXP duckdb_r_allocate(const duckdb::LogicalType &type, duckdb::idx_t nrows);
 void duckdb_r_decorate(const duckdb::LogicalType &type, SEXP dest, bool integer64);
+
+template <typename T, typename... ARGS>
+cpp11::external_pointer<T> make_external(const std::string &rclass, ARGS &&... args) {
+	auto extptr = cpp11::external_pointer<T>(new T(std::forward<ARGS>(args)...));
+	((cpp11::sexp)extptr).attr("class") = rclass;
+	return extptr;
+}
+
+template <typename T, typename... ARGS>
+cpp11::external_pointer<T> make_external_prot(const std::string &rclass, SEXP prot, ARGS &&... args) {
+	auto extptr = cpp11::external_pointer<T>(new T(std::forward<ARGS>(args)...), true, true, prot);
+	((cpp11::sexp)extptr).attr("class") = rclass;
+	return extptr;
+}
