@@ -1,14 +1,16 @@
 import duckdb
 import pytest
+import platform
+import sys
 
 
 @pytest.fixture()
 def tbl_table():
     con = duckdb.default_connection
-    con.execute("drop table if exists tbl")
+    con.execute("drop table if exists tbl CASCADE")
     con.execute("create table tbl (i integer)")
     yield
-    con.execute('drop table tbl')
+    con.execute('drop table tbl CASCADE')
 
 
 class TestRAPIQuery(object):
@@ -117,13 +119,15 @@ class TestRAPIQuery(object):
 
     def test_replacement_scan_recursion(self, duckdb_cursor):
         depth_limit = 1000
-        import sys
 
-        if sys.platform.startswith('win'):
-            # With the default we reach a stack overflow in the CI
+        if sys.platform.startswith('win') or platform.system() == "Emscripten":
+            # With the default we reach a stack overflow in the CI for windows
+            # and also outside of it for Pyodide
             depth_limit = 250
+
         duckdb_cursor.execute(f"SET max_expression_depth TO {depth_limit}")
-        rel = duckdb_cursor.sql('select 42')
-        rel = duckdb_cursor.sql('select * from rel')
-        with pytest.raises(duckdb.BinderException, match=f'Max expression depth limit of {depth_limit} exceeded'):
-            duckdb_cursor.sql('select * from rel')
+        rel = duckdb_cursor.sql('select 42 a, 21 b')
+        rel = duckdb_cursor.sql('select a+a a, b+b b from rel')
+        other_rel = duckdb_cursor.sql('select a from rel')
+        res = other_rel.fetchall()
+        assert res == [(84,)]
